package rand.asset_forfeiture;

import javax.swing.*;
import java.util.HashMap;
import java.awt.event.*;

/** The DisplayPanel is the main user interface to the Asset
 * Forfeiture tool.
 */
public class DisplayPanel extends JPanel implements ActionListener {

    /** An array of column headings for the input/output section, in
     * the order in which they will appear.
     */
    public String[] columnHeadings = null;

    /** The labels for each of the input boxes. */
    private JLabel[] labels = null;

    /** The labels for each of the output rows. */
    private JLabel[] alternateLabels = null;

    /** The JTextFields that are used for all input/output, with
     * [0][0] being the upper left hand field.
     */
    private JTextField[][] textFields = null;

    /** The layout manager for the main input/output panel section. */
    private SpringLayout manager = null;

    /** The only constructor takes an array of ProfitCalculators that
     * are to be evaluated and creates a grid of appropriate size to
     * display all the results for each ProfitCalculator.
     *
     * \param pcs an array of ProfitCalculator-implementing objects
     * that you wish to compare
     */
    public DisplayPanel(ProfitCalculator[] pcs) {
	manager = new SpringLayout();
	setLayout(manager);

		setPreferredSize(new java.awt.Dimension(1100,300));

	columnHeadings = new String[pcs.length + 1];
	columnHeadings[0] = "Inputs";
	for(int i = 0; i < pcs.length; i++)
	    columnHeadings[i+1] = pcs[i].getName();

	labels = new JLabel[variables.values().length];
	alternateLabels = new JLabel[variables.values().length];
	for(variables v: variables.values()) {
	    labels[v.ordinal()] = new JLabel(v.toString());
	    alternateLabels[v.ordinal()] = 
		new JLabel(v.toAlternateString());
	}

	textFields = 
	    new JTextField[columnHeadings.length][];
	for(int c = 0; c < columnHeadings.length; c++)
	    textFields[c] = 
		new JTextField[variables.values().length];

	JComponent longestLabel = rowHeadings();

	for(int c = 0; c < columnHeadings.length; c++)
	    addColumn(c,longestLabel);

	revalidate();
    }

    /** This helper function places all the output row headings in the
     * proper location.
     *
     * \param left the JComponent that should be used as the left-hand
     * boundary for the row headings. Typically, this is one of the input
     * text boxes.
     *
     * \returns the left-hand boundary for the next column of
     * JComponents
     */
    private JComponent alternateRowHeadings(JComponent left) {
	JLabel longest = null;
	for(variables v: variables.values()) {
	    if(longest == null)
		longest = alternateLabels[0];
	    if(alternateLabels[v.ordinal()].getText().length() >
	       longest.getText().length())
		longest = alternateLabels[v.ordinal()];

	    manager.putConstraint(SpringLayout.WEST,
				  alternateLabels[v.ordinal()], 5,
				  SpringLayout.EAST, left);

	    if(v.ordinal() > 0)
		manager.putConstraint(SpringLayout.NORTH,
				      alternateLabels[v.ordinal()],
				      10,
				      SpringLayout.SOUTH,
				      alternateLabels[v.ordinal()-1]);
	    else
		manager.putConstraint(SpringLayout.NORTH,
				      alternateLabels[v.ordinal()],
				      30, SpringLayout.NORTH, this);
	    add(alternateLabels[v.ordinal()]);
	}

	return longest;
    }

    /** This helper method places all the input box headings in the
     * very left-hand column of the panel.
     * 
     * \returns the JComponent that should be the left-hand boundary
     * of the next column of JComponents, typically the input boxes
     * themselves.
     */
    private JComponent rowHeadings() {
	JLabel longestLabel = null;
	for(variables v: variables.values()) {
	    if(longestLabel == null)
		longestLabel = labels[0];
	    if(labels[v.ordinal()].getText().length() >
	       longestLabel.getText().length())
		longestLabel = labels[v.ordinal()];

	    manager.putConstraint(SpringLayout.WEST,
				  labels[v.ordinal()], 5,
				  SpringLayout.WEST,this);
	    if(v.ordinal() > 0)
		manager.putConstraint(SpringLayout.NORTH,
				      labels[v.ordinal()], 10,
				      SpringLayout.SOUTH,
				      labels[v.ordinal()-1]);
	    add(labels[v.ordinal()]);

	}
	manager.putConstraint(SpringLayout.NORTH,
			      labels[0], 30,
			      SpringLayout.NORTH, this);

	JButton rb = null;
	for(variables v: variables.values()) {
	    if(v == variables.diff) continue;
	    rb = new JButton("?");
	    rb.setActionCommand(v.toString());
	    rb.addActionListener(this);
	    add(rb);
	    manager.putConstraint(SpringLayout.WEST,
				  rb, 5, SpringLayout.EAST,
				  longestLabel);
	    manager.putConstraint(SpringLayout.NORTH,
				  rb,-6, SpringLayout.NORTH,
				  labels[v.ordinal()]);
	}

	return rb;
    }

    /** This helper method adds a column of text fields to the main
     * input/output panel. Anything in textFields[c] is overwritten by
     * this method.
     *
     * \param c method adds the cth column of textFields to the panel
     *
     * \param longestLabel the JComponent to use as the left-hand
     * bound for the column. This is only really used for the the first
     * column after the input labels.
     */
    private void addColumn(int c, JComponent longestLabel) {

	JComponent leftBound = null;
	if(c > 1)
	    leftBound = textFields[c-1][0];
	else if(c == 1)
	    leftBound = alternateRowHeadings(textFields[c-1][0]);
	else
	    leftBound = longestLabel;

	JLabel columnHeading = new JLabel(columnHeadings[c]);
	add(columnHeading);
	manager.putConstraint(SpringLayout.NORTH,
			      columnHeading,5,
			      SpringLayout.NORTH, this);
	manager.putConstraint(SpringLayout.WEST,
			       columnHeading, 5,
			       SpringLayout.EAST, leftBound);

	for(variables v: variables.values()) {
	    textFields[c][v.ordinal()] =
		new JTextField("",9);
	    manager.putConstraint(SpringLayout.NORTH,
				  textFields[c][v.ordinal()], -1,
				  SpringLayout.NORTH,
				  labels[v.ordinal()]);
	    manager.putConstraint(SpringLayout.WEST,
				  textFields[c][v.ordinal()],
				  5, SpringLayout.EAST,
				  leftBound);

	    if( (c != 0) || (v != variables.diff))
		add(textFields[c][v.ordinal()]);
	}
    }

    /** This method erases the content of all the textFields, allowing
     * the user to start all over again. There is no undo.
     */
    public void clear() {
	for(int c = 0; c < columnHeadings.length; c++)
	    for(variables v: variables.values())
		textFields[c][v.ordinal()].setText("");
    }

    /** This helper method converts a column name to its ordinal
     * index.
     *
     * \param s the name of the column to be looked up in columnHeadings
     *
     * \returns -1 if the column is not found, its index in
     * columnHeadings if it is.
     */
    private int getIndex(String s) {
	for(Integer i = 0; i < columnHeadings.length; i++)
	    if(columnHeadings[i].contentEquals(s)) {
		return i;
	    }
	return -1;
    }

    /** This is the accessor for the input values. It is written to
     * provide the current values of any of the input or output columns,
     * but it really only has a use (right now) for extracting the input
     * values for transfer to a ProfitCalculator.
     *
     * \param columnName the name of the column whose values you want
     * to extract
     *
     * \returns a mapping from variables to their corresponding values
     */
    public HashMap<variables,Double> getValues(String columnName) {
	HashMap<variables,Double> data = new
	    HashMap<variables,Double>();

	int c = getIndex(columnName);
	for(variables v: variables.values()) {
	    if(v == variables.diff) continue;
	    try{
		String tempS = textFields[c][v.ordinal()].getText();
		tempS = tempS.replace(",","");
		tempS = tempS.replace("$","");
		data.put(v, new Double(tempS));
	    } catch(NumberFormatException e) {
		JOptionPane.
		    showMessageDialog(null,v.toString() +
				      " is not a properly formatted"+
				      " number (" 
				      +textFields[c][v.ordinal()].getText()
				      +").\n\nZero will be"+
				      " used.");
		data.put(v, new Double(0));
	    }
	}

	return data;
    }

    /** A mutator for a whole column's worth of variables at once.
     *
     * \param input a HashMap holding a mapping of the variables to
     * their values.
     * \param cName the name of the column to write these values to
     * 
     */
    public void setValues(HashMap<variables,Double> input,
			  String cName) {
	int c = getIndex(cName);
	Boolean currency = false;
	for(variables v : input.keySet()) {
	    switch(v) {
	    case L1: currency = false;break;
	    case L2: currency = false;break;
	    case L3: currency = false;break;
	    case PL1: currency = true;break;
	    case PL2: currency = true;break;
	    case PL3: currency = true;break;
	    case R: currency = true;break;
	    case rho: currency = true;break;
	    case pi: currency = true;break;
	    case diff: currency=true;break;
	    }
	    textFields[c][v.ordinal()].
		setText(GCalc.print(input.get(v),currency));
	}
    }

    /** The only events for this class are generated by my help
     * buttons, so this method assumes that every ActionEvent is a
     * request for help and launches a HelpDialog with the appropriate
     * message.
     * 
     * \param evt the ActionEvent that this listener caught
     */
    public void actionPerformed(ActionEvent evt) {
	String v = evt.getActionCommand();
	variables myV = variables.fromString(v);
	HelpDialog h = new HelpDialog(myV.getHelp());
	h.setVisible(true);
    }
}
