package rand.asset_forfeiture;

import javax.swing.*;
import java.util.*;

/** This class contains lots of the common functionality for KFixed,
 * KVariable, and Benchmark. It is not meant to be implemented.
 */
public abstract class GCalc implements ProfitCalculator {

    /** Local storage for the values of all the available variables. */
    private HashMap<variables,Double> variableData = null;

    /** Need to store the alternate PL3 outside of variableData
     * because we don't want to create an extra variable for it.
     */
    private Double newPL3 = 0.0;

    /** The only constructor just initializes variableData */
    public GCalc() {
	variableData = new HashMap<variables,Double>();
    }

    // documented in ProfitCalculator
    public abstract Boolean needsVar(variables v);

    // documented in ProfitCalculator
    public abstract String getName();

    /** A nice header for printing out the results */
    public String getTitle() {
	return "---------" + getName() + "----------\n";
    }

    // documented in ProfitCalculator
    public Boolean validate() {
	Boolean valid = true;
	for(variables v: variables.values())
	    valid = valid & ( !needsVar(v) || (variableData.get(v) != null));

	if(!valid)
	    JOptionPane.showMessageDialog(null, "Not all required "+
					  "variables have been "+
					  "entered.");

	return valid;
    }

    /** Tests for valid ranges of the technology parameters. These
     * depend upon the initial inputs, but not upon the optimization.
     * 
     * \param displayBox if true, an error box will pop up if the
     * parameters do not have valid ranges
     *
     * \returns true if the inputs are valid; false otherwise
     */
    protected Boolean validRange(Boolean displayBox) {
	Boolean valid = true;

	valid = valid && (0 < alpha()) && (alpha() < 1);
	if(!valid) System.err.println("alpha not valid");
	valid = valid && (0 < beta()) && (beta() < 1);
	if(!valid) System.err.println("beta not valid");
	valid = valid && (0 < gamma()) && (gamma() < 1);
	if(!valid) System.err.println("gamma not valid");
	valid = valid && (0 < delta()) && (delta() < 1);
	if(!valid) System.err.println("delta not valid");
	valid = valid && (0 <= epsilon()) && (epsilon() < 1);
	if(!valid) System.err.println("epsilon not valid");
	valid = valid &&
	    (alpha()+beta()+gamma()+delta()+epsilon() < 1);
	if(!valid && displayBox)
	    JOptionPane.showMessageDialog(null,
					  "Inputs do not reflect "
					  + "a valid situation.");
	return valid;
    }

    // documented in ProfitCalculator
    public Boolean setVals(HashMap<variables,Double> vals) {

	//copies all entries in m into data, replacing existing values
	variableData.putAll(vals);

	return validate();
    }

    // documented in ProfitCalculator
    public HashMap<variables,Double> getOptimal() {

	HashMap<variables,Double> m = new HashMap<variables,Double>();

	m.put(variables.L1, X1Opt());
	m.put(variables.L2, X4Opt());
	m.put(variables.L3, X3Opt());
	m.put(variables.PL1, PL1());
	m.put(variables.PL2, PL4());
	m.put(variables.PL3, PL3());
	m.put(variables.R, ROpt());
	m.put(variables.rho, rhoOpt());
	m.put(variables.pi, piOpt());
	m.put(variables.diff, pi() - piOpt());

	return m;
    }

    /** Accessor for X1
     *
     * \returns L1 + 1 to avoid problems of having no L1 workers
     */
    protected Double X1() {
	return Math.max(variableData.get(variables.L1),1);
    }

    /** Accessor for X2
     *
     * \returns L2 + 1 to avoid problems of having no L2 workers
     */
    protected Double X2() {
	return Math.max(variableData.get(variables.L2),1);
    }

    /** Accessor for X3
     *
     * \returns L3 + 1 to avoid problems of having no L3 workers
     */
    protected Double X3() {
	return Math.max(variableData.get(variables.L3),1);
    }

    /** Accessor for PL1
     *
     * \returns PL1
     */
    protected Double PL1() {
	return variableData.get(variables.PL1);
    }

    /** Accessor for PL2
     *
     * \returns PL2
     */
    protected Double PL2() {
	return variableData.get(variables.PL2);
    }

    /** Accessor for PL3
     *
     * \returns PL3
     */
    protected Double PL3() {
	return variableData.get(variables.PL3);
    }

    /** Accessor for PL4
     *
     * \returns newPL3
     */
    protected Double PL4() {
	return newPL3;
    }

    /** Accessor for pi
     *
     * \returns pi
     */
    protected Double pi() {
	return variableData.get(variables.pi);
    }

    /** Accessor for R
     *
     * \returns R
     */
    protected Double R() {
	return variableData.get(variables.R);
    }

    /** Accessor for rho
     *
     * \returns rho
     */
    protected Double rho() {
	return variableData.get(variables.rho);
    }

    /** Accessor for Q
     *
     * \returns the calculated Q based on input values
     */
    protected Double Q() {

	Double t1 = Math.pow(K(),alpha());
	Double t2 = Math.pow(rho(),epsilon());
	Double t3 = Math.pow(X1(),beta());
	Double t4 = Math.pow(X2(),gamma());
	Double t5 = Math.pow(X3(),delta());

	return t1*t2*t3*t4*t5;
    }

    /** User-readable formula for Q
     *
     * \returns a string with the formula used to calculate Q
     */
    protected String QString() {
	return "K^a * rho^e * X1^b * X2^g * X3^d";
    }

    /** Accessor for K
     *
     * \returns the calculated K based on input values
     */
    protected Double K() {
	return
	    R()-pi()-rho()-PL1()*X1()-PL2()*X2()-PL3()*X3();
    }

    /** User-readable formula for K
     *
     * \returns a string with the formula used to calculate K
     */
    protected String KString() {
	return "R - pi - rho - PL1 * X1 - PL2 * X2 - PL3 * X3";
    }

    /** Accessor for alpha
     *
     * \returns the calculated alpha based on input values
     */
    protected Double alpha() {
	return K()/R();
    }

    /** User-readable formula for alpha
     *
     * \returns a string with the formula used to calculate alpha
     */
    protected String alphaString() {
	return "K / R";
    }

    /** Accessor for beta
     *
     * \returns the calculated beta based on input values
     */
    protected Double beta() {
	return X1()*PL1()/R();
    }

    /** User-readable formula for beta
     *
     * \returns a string with the formula used to calculate beta
     */
    protected String betaString() {
	return "X1 * PL1 / R";
    }

    /** Accessor for gamma
     *
     * \returns the calculated value of gamma based on input values
     */
    protected Double gamma() {
	return X2()*PL2()/R();
    }

    /** User-readable formula for gamma
     *
     * \returns a string with the formula used to calculate gamma
     */
    protected String gammaString() {
	return "X2 * PL2 / R";
    }

    /** Accessor for delta
     *
     * \returns the calculated value of delta based on input values
     */
    protected Double delta() {
	return X3()*PL3()/R();
    }

    /** User-readable formula for delta
     *
     * \returns a string with the formula used to calculate delta
     */
    protected String deltaString() {
	return "X3 * PL3 / R";
    }

    /** Accessor for epsilon
     *
     * \returns the calculated value of epsilon based on input values
     */
    protected Double epsilon() {
	return rho()/R();
    }

    /** User-readable formula for epsilon
     *
     * \returns a string with the formula used to calculate epsilon
     */
    String epsilonString() {
	return "rho / R";
    }

    /** Accessor for Pq
     *
     * \returns the calculated value of Pq based on input values
     */
    protected Double Pq() {
	return R()/Q();
    }

    /** User-readable formula for Pq
     *
     * \returns a string with the formula used to calculate Pq
     */
    protected String PqString() {
	return "R / Q";
    }

    /** Abstract methods */
    //@{
    /** Accessor for optimal K
     *
     * \returns the optimized value of K
     */
    abstract protected Double KOpt();

    /** User-readable formula for optimal K
     *
     * \returns a string with the formula used to optimize K
     */
    abstract protected String KOptString();

    /** Accessor for optimal phi3
     *
     * \returns the optimized value of phi3
     */
    abstract protected Double phi3();

    /** User-readable formula for optimal phi3
     *
     * \returns a string with the formula used to optimize phi3
     */
    abstract protected String phi3String();

    /** Accessor for optimal X3
     *
     * \returns the optimized value of X3, unless X3 < 1, in which
     * case it returns one, since real labor = X3-1
     */
    abstract protected Double X3Opt();

    /** User-readable formula for optimal X3
     *
     * \returns a string with the formula used to optimize X3
     */
    abstract protected String X3OptString();
    //@}

    /** Accessor for omega
     *
     * \return the calculated value of omega
     */
    protected Double omega() {
	return PL3()/PL1()*beta()/delta();
    }

    /** User-readable formula for omega
     *
     * \returns a string with the formula for omega
     */
    protected String omegaString() {
	return "PL3 / PL1 * beta / delta";
    }

    /** Accessor for optimal X1
     *
     * \returns the optimal value of X1, unless that optimum is less
     * than 1, in which case it returns 1 (real labor = X1 - 1)
     */
    protected Double X1Opt() {
	return Math.max(omega() * X3Opt(), 1);
    }

    /** User-readable formula for optimal X1
     *
     * \returns a string containing the formula used to optimize X1
     */
    protected String X1OptString() {
	return "omega * X3*";
    }

    /** Accessor for X4
     *
    * \returns the optimal value of X4, unless that optimum is less
     * than 1, in which case it returns 1 (real labor = X1 - 1)
     */
    protected Double X4Opt() {
	return Math.max(1, PL3() / PL4() * X3Opt() * gamma() / delta());
    }

    /** User-readable formula for X4
     *
     * \returns a string containing the formula used to optimize X4
     */
    protected String X4OptString() {
	return "PL3 / PL4 * X3 * g / d";
    }

    /** Accessor for optimal rho
     *
     * \returns the optimal value of rho
     */
    protected Double rhoOpt() {
	return PL3() * X3Opt() * epsilon() / delta();
    }

    /** User-readable formula for rho
     *
     * \returns a string containing the formula used to optimize rho
     */
    protected String rhoOptString() {
	return "PL3 * X3* * e / d";
    }

    /** Accessor for optimal Q
     *
     * \returns the optimized value of Q
     */
    protected Double QOpt() {
	Double t1 = Math.pow(KOpt(),alpha());
	Double t2 = Math.pow(rhoOpt(),epsilon());
	Double t3 = Math.pow(X1Opt(),beta());
	Double t4 = Math.pow(X4Opt(),gamma());
	Double t5 = Math.pow(X3Opt(),delta());

	return t1*t2*t3*t4*t5;
    }

    /** User-readable formula for the optimal Q
     *
     * \returns a string containing the formula used to optimize Q
     */
    protected String QOptString() {
	return "K*^a * rho*^e * X1*^b * X4*^g * X3*^d";
    }

    /** Accessor for the optimal pi
     *
     * \returns the optimal value of pi
     */
    protected Double piOpt() {
	Double t1 = ROpt();
	Double t2 = KOpt();
	Double t3 = rhoOpt();
	Double t4 = PL1()*X1Opt();
	Double t5 = PL3()*X3Opt();
	Double t6 = PL4() * X4Opt();

	return t1 - t2 - t3 - t4 - t5 - t6;
    }

    /** User-readable formula for the optimal pi
     *
     * \returns a string containing the formula used to optimize pi
     */
    protected String piOptString() {
	return "Pq * Q*"
	    +"- K* - rho* - PL1 * X1* - PL3 * X3* - PL4 * X4*";
    }

    /** Accessor for the optimal R
     *
     * \returns the optimized value of R
     */
    protected Double ROpt() {
	return Pq() * QOpt();
    }

    /** Mutator to set an alternative value for PL3 based on
     * BLS or domain knowledge.
     *
     * \param p the alternate PL3
     */
    public void setNewPL3(double p) {
	newPL3 = p;
    }

    /** A method to standardize the look and feel of doubles in the
     * DisplayPanel as well as the text outputs.
     *
     * \param d the Double to be formatted
     * \param currency setting this to true will treat d as a dollar
     * value instead of a raw number
     *
     * \returns a string that represents d in a standard fasion. If
     * currency is true, then the string will have a dollar sign in
     * front and be limited to two decimal places. If currency is
     * false, then d will be formatted using the standard C notation
     * as "%,8.4f".
     */
    public static String print(Double d, Boolean currency) {
	int len;
	String s = "";
	String p = "4";
	if(currency) {
	    s += "$";
	    p = "2";
	}
	s += String.format("%,8."+p+"f", d);

    	return s;
    }

    /** An alternative to print(Double, Boolean) that assumes d is not
     * a currency.
     *
     * \param d the number to be formatted
     *
     * \returns a string representing d as above.
     */
    public static String print(Double d) {
	return print(d, false);
    }

    //documented in ProfitCalculator
    public void calculateBase(JTextArea forDebuggers) {

	if(!validRange(true))
	    return;

	printInputs(forDebuggers);

	forDebuggers.append("First, the world with illegal workers\n");
	forDebuggers.append("-------------------------------------\n");

	forDebuggers.append( "alpha (a) =\t" + print(alpha()) + "\t"
			     + alphaString() + "\n");

	forDebuggers.append( "beta (b) =\t" + print(beta()) + "\t" +
			     betaString() + "\n");

	forDebuggers.append( "gamma (g) =\t" + print(gamma()) + "\t"
			     + gammaString() + "\n");

	forDebuggers.append( "delta (d) =\t" + print(delta()) + "\t"
			     + deltaString() + "\n");

	forDebuggers.append( "epsilon (e) =\t" +
			     print(epsilon()) + "\t" +
			     epsilonString() + "\n");

	forDebuggers.append("Pq = \t" + print(Pq(),true) + "\t" +
			    PqString() + "\n");
	forDebuggers.append("Q =\t" + print(Q()) + "\t"
			    + QString() + "\n");
	forDebuggers.append( "K =\t" + print(K(),true) + "\t"
			     + KString() + "\n");
    }

    /** A utility function that prints the values of all the input
     * parameters.
     * 
     * \param D the JTextArea to print to
     */
    private void printInputs(JTextArea D) {

	D.append(getTitle());
	D.append("Inputs\n------------------------------\n");
	D.append("L1 =\t" + print(variableData.get(variables.L1)) + "\n");
	D.append("L2 =\t" + print(variableData.get(variables.L2)) + "\n");
	D.append("L3 =\t" + print(variableData.get(variables.L3)) + "\n");
	D.append("X1 =\t" + print(X1()) + "\n");
	D.append("X2 =\t" + print(X2()) + "\n");
	D.append("X3 =\t" + print(X3()) + "\n");
	D.append("PL1 =\t" + print(PL1(),true) + "\n");
	D.append("PL2 =\t" + print(PL2(),true) + "\n");
	D.append("PL3 =\t" + print(PL3(),true) + "\n");
	D.append("R =\t" + print(R(),true) + "\n");
	D.append("pi =\t" + print(pi(),true) + "\n");
	D.append("rho =\t" + print(rho(),true) + "\n\n");
    }

    //documented in ProfitCalculator
    public void calculateExcursion(JTextArea forDebuggers) {
	if(!validRange(false))
	    return;

	forDebuggers.append( "\n" + getTitle() + " without illegal workers\n");
	forDebuggers.append("-------------------------------------\n");

	forDebuggers.append("PL1 =\t" + print(PL1(),true) + "\n");
	forDebuggers.append("PL4 =\t" + print(PL4(),true) + "\n");
	forDebuggers.append("PL3 =\t" + print(PL3(),true) + "\n");
	forDebuggers.append("Omega =\t" + print(omega()) +
			    "\t" + omegaString() + "\n");
	forDebuggers.append("phi3 =\t" + print(phi3()) + "\t" +
			    phi3String() + "\n");

	forDebuggers.append("K* =\t" + print(KOpt(),true) + "\t" +
			    KOptString() + "\n");
	forDebuggers.append("X1* =\t" + print(X1Opt()) +
			    "\t" + X1OptString() + "\n");
	forDebuggers.append("X4* =\t" + print(X4Opt()) + "\t" +
			    X4OptString() + "\n");
	forDebuggers.append("X3* =\t" + print(X3Opt()) + "\t" +
			    X3OptString() + "\n");

	forDebuggers.append("Q* =\t" + print(QOpt()) + "\t" +
			    QOptString() + "\n");

	forDebuggers.append("pi** =\t" + print(piOpt(),true) + "\t" +
			    piOptString() + "\n\n");
    }


}
