package rand.asset_forfeiture;

import java.util.HashMap;
import javax.swing.JTextArea;

/** An interface that lays out the requirements for a class to
 * calculate the profits derived from hiring illegal immigrants.
 */
public interface ProfitCalculator {

    /** This method will provide the inputs to the calculation in the
     * form of a HashMap mapping an available variable to its
     * value. In general, variables not specified are missing, not
     * assumed to be zero. This is to allow flexibility in allowing some
     * calculations to use only a subset of the available variables.
     * 
     * \param vals the HashMap containing the variable-value mappings
     * 
     * \returns true if the inputs are valid for this particular
     * calculator
     */
    public Boolean setVals(HashMap<variables,Double> vals);

    /** This method runs any input validation available for the
     * ProfitCalculator. This method should check things such as all
     * the required variables being present, values being within
     * predetermined ranges (e.g. wages >= 0), etc.
     * 
     * \returns true if the ProfitCalculator is in a valid state,
     * false otherwise.
     */
    public Boolean validate();

    /** Does the calculations for the world as described, determining
     * the values of any parameters needed for the excursion to a
     * legals-only framework.
     * 
     * \param forDebuggers append any relevant debugging output to
     * this JTextArea for user-level debugging and understanding.
     */
    public void calculateBase(JTextArea forDebuggers);

    /** Does the calculations for the world without illegal
     * immigrants, including what the profits of the firm would be, how
     * many workers of each type would be hired, etc.
     * 
     * \param forDebuggers append any relevant debugging output to
     * this JTextArea for user-level debugging and understanding.
     */
    public void calculateExcursion(JTextArea forDebuggers);
    
    /** The name of this particular ProfitCalculator.
     * 
     * \returns a string with the name of the calculator. Does not
     * have to be unique.
     */
    public String getName();

    /** Determines whether this implementation of ProfitCalculator
     * requires a particular variable for its calculations.
     * 
     * \param v the variable in question
     * 
     * \returns true if the variable is required; false if it is not
     */
    public Boolean needsVar(variables v);

    /** A method for updating the value for PL3 used in
     * calculateExcursion based on some more detailed knowledge that the
     * user has of the case in question.
     * 
     * \param p the new value of PL3
     */
    public void setNewPL3(double p);

    /** While calculateExcursion does the calculations, this method
     * returns the results, so you don't have to do both simultaneously.
     * 
     * \returns a HashMap with the optimal values of the relevant
     * variables
     */
    public HashMap<variables,Double> getOptimal();
}