package rand.asset_forfeiture;

import javax.swing.*;
import java.util.HashMap;
import java.awt.event.*;
import java.awt.*;
import javax.swing.table.*;
import javax.swing.event.*;

/** This is the main class you need to instantiate to run the Wage
 * Calculation Wizard. It will launch a dialog box with a
 * spreadsheet. Enter the appropriate expense and hours worked
 * information and the wizard will calculate the FTE and yearly
 * expense per FTE required. Click OK and it will dump those numbers
 * in to the main DisplayPanel, overriding any previous values.
 */
public class WageWizard 
    extends JDialog
    implements ActionListener, TableModelListener {

    /** \name Timekeeping Constants
     * A group of known constants for calculating the number of hours
     * in a given period of time.
     */
    //@{
    public static final int HOURS_PER_DAY = 8;
    public static final int DAYS_PER_WEEK = 5;
    /** Calculated number of work hours per week for FTE
     * \param HOURS_PER_DAY number of work hours in a day
     * \param DAYS_PER_WEEK number of workdays per week
     */
    public static final int HOURS_PER_WEEK = 
	HOURS_PER_DAY *	DAYS_PER_WEEK;
    public static final int WEEKS_PER_YEAR = 52;
    /** Calculated number of work hours per year for FTE
     * \param HOURS_PER_WEEK number of work hours in a week
     * \param WEEKS_PER_YEAR number of work weeks in a year
     */
    public static final int HOURS_PER_YEAR = 
	HOURS_PER_WEEK * WEEKS_PER_YEAR;
    public static final int MONTHS_PER_YEAR = 12;
    /** Calculated number of work hours per month for FTE
     * \param HOURS_PER_YEAR number of work hours per year
     * \param MONTHS_PER_YEAR number of work months per year
     */
    public static final int HOURS_PER_MONTH = 
	HOURS_PER_YEAR / MONTHS_PER_YEAR;
    /** Calculated number of workdays per year for FTE
     * \param DAYS_PER_WEEK number of work days per week
     * \param WEEKS_PER_YEAR number of work weeks per year
     */
    public static final int DAYS_PER_YEAR =
	DAYS_PER_WEEK*WEEKS_PER_YEAR;
    //@}

    /** The default total benefits overhead, as a fraction of base
     * salary.
     */
    public static final double TOTAL_BENEFITS = 0.412;

    /** The portion of the total benefits overhead due to payroll
     * taxes.
     */
    public static final double SS_MEDICARE = 0.0765;

    /** This enum describes the column headings for the WageWizard
     * spreadsheet.
     */
    public enum columnHeadings {
	nWorkers, //!< The number of workers in the row
	    workerType, //!< The workerTypes
	    wage, //!< The amount of money paid per period
	    wagePeriod, //!< The period used for yearly wages
	    wageType, //!< The wageTypes
	    benefits, //!< The benefits overhead
	    /** Instead of entering all the expense parameters, you
	     * can just enter the total wage bill per worker. This
	     * will be modified to determine the wage bill per FTE if
	     * necessary.
	     */
	    wageBill,
	    hpd, //!< The number of hours per day worked
	    dpw, //!< The number of days per week worked
	    wpy, //!< The number of weeks per year worked
	    fte, //!< An output: number of full time equivalent workers
	    expenses, //!< An output: yearly expense per FTE
	    realCost; //!< An output: total yearly expense

	/** Give the enum a custom string converter in order to have
	 * more user-friendly column headings.
	 */
	public String toString() {
	    switch(this) {
	    case nWorkers: return "Number of Workers";
	    case workerType: return "Worker Type";
	    case wage: return "Wage ($)";
	    case wagePeriod: return "Payment Period";
	    case wageType: return "Wage Type";
	    case benefits: return "Benefit Overhead (decimal)";
	    case wageBill: return "Wage Bill per Worker";
	    case hpd: return "Hours per Day";
	    case dpw: return "Days per Week";
	    case wpy: return "Weeks per Year";
	    case fte: return "FTE Workers";
	    case expenses: return "Yearly Expense per FTE";
	    case realCost: return "Actual Cost";
	    }

	    return "Unknown columnHeadings";
	}
    }

    /** An enum for the possible pay periods. These show up in the
     * combo box selection when using the WageWizard.
     */
    private enum timePeriods{Hourly, Daily, Weekly, Monthly, Yearly};

    /** These are the three types of workers you can choose from in
     * the WageWizard. Ln is our internal notation.
     */
    private enum workerTypes{L1, L2, L3;
	/** A custom toString method for more user-friendly combo box
	 * selections.
	 */
	public String toString() {
	    switch(this) {
	    case L1: return "Legal Skilled";
	    case L2: return "Illegal Unskilled";
	    case L3: return "Legal Unskilled";
	    }
	    return "Unknown workerType";
	}
    }

    /** Each row in the WageWizard can describe either hourly or
     * salaried employees. The big difference is that salaried employees
     * are replaced one-to-one when calculating FTE, while hourly
     * employees are replaced at the rate of one FTE per 8 hours of labor
     * per day.
     */
    private enum wageTypes{Hourly, Salaried};

    /** This model holds all the actual data for the WageWizard.
     */
    private static WageTableModel wtm = null;

    /**\name Interface Buttons
     */
    //@{
    /** OK means you are done, so save the results to the DisplayPanel
     */
    private JButton okB = null;
    /** Cancel exits without saving
     */
    private JButton cancelB = null;
    /** Adds another row to the table in case you run out
     */
    private JButton addRowB = null;
    /** Removes the active row as a simple alternative to zeroing all
     * the entries by hand
     */
    private JButton removeRowB = null;
    //@}

    /** This variable keeps track of whether the data stored in wtm
     * are valid or not.
     */
    private Boolean valid = false;

    /** This is the table that is the main interface for the
     * WageWizard. It is kind of like a spreadsheet, except you need to
     * implement all the functionality in java instead of as formulae in
     * the cells.
     */
    private JTable myTable;

    /** This class only gets one constructor, and this is it.
     *
     * \param owner The Frame that owns this JDialog; can be null
     * \param title The title that shows in the window bar
     * \param modal If modal, no other windows in this application can
     * be accessed until you are done with this one
     */
    public WageWizard(Frame owner, String title, boolean modal) {
	super(owner,title,modal);

	setLayout(new BorderLayout());

	if(wtm == null) {
	    wtm = new WageTableModel(columnHeadings.values());
	    for(int i=0; i < 5; i++)
		addRow();
	}

	myTable = new JTable(wtm);
	myTable.getModel().addTableModelListener(this);

	// Need to do some special setup to get the combo boxes to
	// show

	int[] textColumns = {columnHeadings.nWorkers.ordinal(),
			     columnHeadings.wage.ordinal(),
			     columnHeadings.benefits.ordinal(),
			     columnHeadings.hpd.ordinal(),
			     columnHeadings.dpw.ordinal(),
			     columnHeadings.wpy.ordinal(),
			     columnHeadings.fte.ordinal(),
			     columnHeadings.expenses.ordinal(),
			     columnHeadings.realCost.ordinal(),
			     columnHeadings.wageBill.ordinal()};
	int[] workTypeColumns = {columnHeadings.workerType.ordinal()};
	int[] periodColumns = {columnHeadings.wagePeriod.ordinal()};
	int[] wageTypeColumns = {columnHeadings.wageType.ordinal()};

	TableColumn col;

	for(int i: textColumns) {
	    col = myTable.getColumnModel().getColumn(i);
	    col.setCellEditor(new MyTextBoxEditor());
	    col.setCellRenderer(new MyTextBoxRenderer());
	}

	for(int i: workTypeColumns) {
	    col = myTable.getColumnModel().getColumn(i);
	    col.setCellEditor(new 
			      MyComboBoxEditor(workerTypes.values()));
	    col.setCellRenderer(new
				MyComboBoxRenderer(workerTypes.values()));
	}

	for(int i: periodColumns) {
	    col = myTable.getColumnModel().getColumn(i);
	    col.setCellEditor(new
			      MyComboBoxEditor(timePeriods.values()));
	    col.setCellRenderer(new
				MyComboBoxRenderer(timePeriods.values()));
	}

	for(int i:wageTypeColumns) {
	    col = myTable.getColumnModel().getColumn(i);
	    col.setCellEditor(new
			      MyComboBoxEditor(wageTypes.values()));
	    col.setCellRenderer(new
				MyComboBoxRenderer(wageTypes.values()));
	}

	JScrollPane jsp = new
	JScrollPane(myTable,ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
		    ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
	add(jsp,BorderLayout.CENTER);
	wtm.pack(myTable);
	GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
	Rectangle bounds = env.getMaximumWindowBounds();
	Dimension d = new Dimension(Math.round(bounds.width*0.9f),
				    Math.round( bounds.height*0.7f));
	myTable.setPreferredScrollableViewportSize(d);

	JPanel p = new JPanel();

	addRowB = new JButton("Add Another Row");
	addRowB.addActionListener(this);
	p.add(addRowB);

	removeRowB = new JButton("Clear the current row");
	removeRowB.addActionListener(this);
	p.add(removeRowB);

	okB = new JButton("OK");
	okB.addActionListener(this);
	p.add(okB);

	cancelB = new JButton("Cancel");
	cancelB.addActionListener(this);
	p.add(cancelB);

	add(p,BorderLayout.SOUTH);
	pack();
    }

    /** This is the event handler for clicking on the Interface
     * Buttons.
     *
     * \param evt The ActionEvent that needs to be processed
     */
    public void actionPerformed(ActionEvent evt) {
	JButton source = (JButton) evt.getSource();
	if(source == okB) {
	    valid = true;
	    dispose();
	} else if(source == cancelB) {
	    valid = false;
	    dispose();
	} else if(source == addRowB) {
	    addRow();
	} else if(source == removeRowB) {
	    removeRow();
	}
    }

    /** Busy is true when the program is editing cells. Whenever the
     * user changes something, we want to recalculate the results in that
     * row. However, putting the new results in to the output cells
     * triggers another TableModelEvent, so we want to ignore those
     * events generated by the internal program logic.
     */
    private Boolean busy = false;

    /** This is an event handler to process TableModelEvents whenever
     * the wtm is changed. This method uses the busy attribute to
     * determine whether the event was triggered by the user or by the
     * program itself.
     *
     * \param evt The TableModelEvent to be processed
     */
    public void tableChanged(TableModelEvent evt) {
	if(TableModelEvent.UPDATE != evt.getType())
	    return;

	if(! busy) {
	    int R = evt.getFirstRow();
	    calculateRow(R);
	}
    }

    /** This method examines all the input columns of the given row
     * and updates the corresponding output columns so that the results
     * shown always correspond to the current inputs.
     *
     * \param row The index to determine which row needs to be
     * processed
     */
    private void calculateRow(int row) {

	busy = true;

	Object o = wtm.getValueAt(row,
				  columnHeadings.nWorkers.ordinal());
	double numW = new Double( (String) o);

	o = wtm.getValueAt(row,
			   columnHeadings.wage.ordinal());
	double wages = new Double( (String) o);

	double hours = new Double( (String) wtm.
				  getValueAt(row, 
					     columnHeadings.hpd.ordinal()));
	double days = new Double( (String) wtm.
				  getValueAt(row,
					     columnHeadings.dpw.ordinal()));
	double weeks = new Double( (String) wtm.
				   getValueAt(row,
					      columnHeadings.wpy.ordinal()));

	double benies = new Double( (String) wtm.
				    getValueAt(row, 
					       columnHeadings.benefits.ordinal()));

	double wageBill = new Double( (String) wtm.
				      getValueAt(row,
						 columnHeadings.wageBill.ordinal()));

	timePeriods payPeriod = (timePeriods) wtm.
	    getValueAt(row, columnHeadings.wagePeriod.ordinal());
	wageTypes wType = (wageTypes) wtm.
	    getValueAt(row, columnHeadings.wageType.ordinal());

	double wagePerHour = 0;
	double hoursPerYear = hours*days*weeks;

	switch(payPeriod) {
	case Hourly: wagePerHour = wages;break;
	case Daily: wagePerHour = wages/HOURS_PER_DAY;break;
	case Weekly: wagePerHour = wages/HOURS_PER_WEEK;break;
	case Monthly: wagePerHour = wages/HOURS_PER_MONTH;break;
	case Yearly: wagePerHour = wages/HOURS_PER_YEAR;break;
	}

	double fteCost = Math.max(wagePerHour * HOURS_PER_YEAR * (1 +
								  benies),
				  wageBill);

	wtm.setValueAt(fteCost, row,
		       columnHeadings.expenses.ordinal());

	if(wType == wageTypes.Hourly)
	    wtm.setValueAt(hoursPerYear/HOURS_PER_YEAR * numW,
			   row, columnHeadings.fte.ordinal());
	else if(wType == wageTypes.Salaried)
	    wtm.setValueAt(numW, row, columnHeadings.fte.ordinal());

	if(wType == wageTypes.Hourly)
	    wtm.setValueAt(fteCost*hoursPerYear/HOURS_PER_YEAR*numW,
			   row, columnHeadings.realCost.ordinal());
	else if(wType == wageTypes.Salaried)
	    wtm.setValueAt(fteCost * numW,
			   row, columnHeadings.realCost.ordinal());

	busy = false;
    }

    /** This method returns a HashMap of the final results. When the
     * WageWizard is valid (the user clicked OK instead of Cancel), the
     * HashMap has entries for all the non-zero variables. When invalid,
     * the HashMap is empty.
     *
     * \returns a HashMap containing the results of the WageWizard
     * calculations
     */
    public HashMap<variables, Double> getValues() {

	HashMap<variables, Double> data = 
	    new HashMap<variables, Double>();

	if(valid) {
	    double l1 = 0;
	    double l2 = 0;
	    double l3 = 0;
	    double pl1 = 0;
	    double pl2 = 0;
	    double pl3 = 0;

	    for(int r=0; r < wtm.getRowCount(); r++) {
		double fte = (Double) wtm.
		    getValueAt(r, columnHeadings.fte.ordinal());
		double wage = (Double) wtm.
		    getValueAt(r, columnHeadings.expenses.ordinal());
		workerTypes wt = (workerTypes) wtm.
		    getValueAt(r, columnHeadings.workerType.ordinal());
		switch(wt) {
		case L1:
		    l1 += fte;
		    pl1 += fte * wage;
		    break;
		case L2:
		    l2 += fte;
		    pl2 += fte * wage;
		    break;
		case L3:
		    l3 += fte;
		    pl3 += fte * wage;
		    break;
		}
	    }

	    if(l1 > 0) {
		pl1 = pl1/l1;
		data.put(variables.L1, l1);
		data.put(variables.PL1, pl1);
	    }

	    if(l2 > 0) {
		pl2 = pl2/l2;
		data.put(variables.L2, l2);
		data.put(variables.PL2, pl2);
	    }

	    if(l3 > 0) {
		pl3 = pl3/l3;
		data.put(variables.L3, l3);
		data.put(variables.PL3, pl3);
	    }
	}

	return data;
    }

    /** A simple array that holds the initial values used for each row
     * of the WageWizard.
     */
    private Object[] 
	initialData = {"0",
		       workerTypes.L1,
		       "0",
		       timePeriods.Hourly,
		       wageTypes.Hourly,
		       (new
			Double(TOTAL_BENEFITS+SS_MEDICARE)).toString(),
		       "0",
		       "0",
		       "0",
		       "0",
		       0.0,
		       0.0,
		       0.0};

    /** This method adds one more editable row to wtm.
     */
    public void addRow() {
	wtm.addRow(initialData);
    }

    /** This method stops the edit process on a cell, because things
     * get very screwed up when you are currently editing a cell and
     * decide to remove the row that cell is in.  Essentially, it will
     * take the value you have typed and copy it in to that column in the
     * row that replaces the one you removed (i.e. the row below the
     * current one).
     *
     * \returns true if the process succeeds, false if there is an
     * exception thrown
     */
    private boolean stopCellEditing() {
	try {
	    int c = myTable.getEditingColumn();
	    if (c > -1) {
		TableCellEditor cellEditor = myTable.
		    getColumnModel().getColumn(c).getCellEditor();
		if ( cellEditor == null ) {
		    cellEditor = myTable.
			getDefaultEditor( wtm.getColumnClass(c) );
		}
		if ( cellEditor != null ) { cellEditor.stopCellEditing(); }
	    }
	} catch (Exception e) {
	    return false;
	}
	return true;
    }

    /** This method removes the currently selected row from the
     * WageWizard.
     */
    public void removeRow() {
	stopCellEditing();

	int rSelect = myTable.getSelectedRow();
	if(rSelect > -1)
	    wtm.removeRow(rSelect);
	
    }

    /** Here is just a little class to properly display the
     * combo boxes within cells of the WageWizard.
     */
    public class MyComboBoxRenderer 
	extends JComboBox 
	implements TableCellRenderer {

	/** The constructor just calls its super; nothing fancy.
	 * 
	 * \param items an object array of the options; see JComboBox
	 * for details
	 */
        public MyComboBoxRenderer(Object[] items) {
            super(items);
        }
    
	/** This method is required to implement TableCellRenderer and
	 * takes care of setting the proper appearance.
	 * 
	 * \param table the JTable in question
	 * \param value the value selected by the user in the combo
	 * box
	 * \param isSelected true if something has been selected
	 * \param hasFocus true if the cell has focus
	 * \param row the row of the cell in question
	 * \param column the column of the cell in question
	 * 
	 * \returns this instance of MyComboBoxRenderer
	 */
        public Component getTableCellRendererComponent(JTable table, Object value,
                boolean isSelected, boolean hasFocus, int row, int column) {
            if (isSelected) {
                setForeground(table.getSelectionForeground());
                super.setBackground(table.getSelectionBackground());
            } else {
                setForeground(table.getForeground());
                setBackground(table.getBackground());
            }
    
            // Select the current value
            setSelectedItem(value);
            return this;
        }
    }
    
    /** A class to handle choosing items from a combo box inside of a
     * JTable.
     */
    public class MyComboBoxEditor extends DefaultCellEditor {
	/** The constructor just calls the super, but with a JComboBox
	 * rather than an array of items.
	 * 
	 * \param items the elements to list in the combo box
	 */
        public MyComboBoxEditor(Object[] items) {
            super(new JComboBox(items));
        }
    }

}
